struct TEditorPos
{
	TEditorPos()
	{
		Default();
	}
	int Row, Col;
	int TopRow, LeftCol;
	void Default (void)
	{
		Row = Col = TopRow = LeftCol = -1;
	}
};

static TEditorPos EditorGetPos (void)
{
	TEditorPos	r;
	EditorInfo	ei;
	Info.EditorControl (ECTL_GETINFO, &ei);
	r.Row = ei.CurLine;
	r.Col = ei.CurPos;
	r.TopRow = ei.TopScreenLine;
	r.LeftCol = ei.LeftPos;
	return (r);
}

static void EditorSetPos (TEditorPos pos)
{
	EditorSetPosition sp;
	sp.CurLine = pos.Row;
	sp.CurPos = pos.Col;
	sp.TopScreenLine = pos.TopRow;
	sp.LeftPos = pos.LeftCol;
	sp.CurTabPos = sp.Overtype = -1;
	Info.EditorControl (ECTL_SETPOSITION, &sp);
	Info.EditorControl (ECTL_TURNOFFMARKINGBLOCK, NULL);
}

void EditorSetPos (int line, int col, int topline = -1, int leftcol = -1)
{
	EditorSetPosition sp;
	sp.CurLine = (line < -1) ? -1 : line;
	sp.CurPos = (col < -1) ? -1 : col;
	sp.TopScreenLine = topline;
	sp.LeftPos = leftcol;
	sp.CurTabPos = sp.Overtype = -1;
	Info.EditorControl (ECTL_SETPOSITION, &sp);
	Info.EditorControl (ECTL_TURNOFFMARKINGBLOCK, NULL);
}

static int EditorGetCodeTable (void)
{
	EditorInfo	ei;
	Info.EditorControl (ECTL_GETINFO, &ei);
	return ((ei.TableNum < 0) ? (ei.AnsiMode + 1) : (ei.TableNum + 3));
}

static void EditorSetCodeTable (int iCodeTable = 1)
{
	EditorSetParameter	etp;
	ZeroMemory (&etp, sizeof (etp));
	etp.Type = ESPT_CHARTABLE;
	etp.Param.iParam = iCodeTable;
	Info.EditorControl (ECTL_SETPARAM, &etp);
}

static void EditorGetStr (EditorGetString *gs, int line = -1)
{
	gs->StringNumber = line;
	Info.EditorControl (ECTL_GETSTRING, gs);
}

static void EditorSetStr (char *src, int line = -1)
{
	EditorSetString st;
	st.StringNumber = line;
	st.StringText = src;
	st.StringEOL = 0;
	st.StringLength = lstrlen (src);
	Info.EditorControl (ECTL_SETSTRING, &st);
}

static void EditorSaveSelected (void)
{
	int					vblock;
	int					selstart = 0;
	int					sellength = 0;
	EditorInfo	ei;
	Info.EditorControl (ECTL_GETINFO, &ei);
	vblock = (ei.BlockType == BTYPE_COLUMN) ? 1 : 0;
	selstart = ei.BlockStartLine;
	if (ei.BlockType != BTYPE_NONE)
	{
		int		curpos;
		char	filename[NM];
		lstrcpy (filename, TEMP_TT_TMP);
		FSF.ExpandEnvironmentStr (filename, filename, sizeof (filename));

		FILE						*file = fopen (filename, "wb");
		EditorGetString egs;
		fwrite (&vblock, 1, sizeof (int), file);
		curpos = selstart;
		do
		{
			EditorGetStr (&egs, curpos++);
			if (egs.SelStart != -1)
			{
				sellength++;
			}
		} while ((egs.SelStart != -1) && (curpos < ei.TotalLines));
		fwrite (&sellength, 1, sizeof (int), file);
		curpos = selstart;
		do
		{
			EditorGetStr (&egs, curpos++);
			if (egs.SelStart != -1)
			{
				int len;
				if (egs.SelEnd != -1)
				{
					char	tmp = ' ';
					int		truelen = ((egs.StringLength - egs.SelStart) > 0) ? (egs.StringLength - egs.SelStart) : 0;
					len = egs.SelEnd - egs.SelStart;
					fwrite (&len, 1, sizeof (int), file);
					fwrite (egs.StringText + egs.SelStart, (truelen < len) ? truelen : len, sizeof (char), file);
					truelen = (truelen < len) ? len - truelen : 0;
					for (int i = 0; i < truelen; i++) fwrite (&tmp, 1, sizeof (char), file);
				}
				else
				{
					len = egs.StringLength - egs.SelStart;
					fwrite (&len, 1, sizeof (int), file);
					fwrite (egs.StringText + egs.SelStart, len, sizeof (char), file);
				}
			}
		} while ((egs.SelStart != -1) && (curpos < ei.TotalLines));
		fclose (file);
	}
}

static void EditorSaveAll (void)
{
	int		vblock = 0;
	char	filename[NM];
	lstrcpy (filename, TEMP_TT_TMP);
	FSF.ExpandEnvironmentStr (filename, filename, sizeof (filename));

	FILE				*file = fopen (filename, "wb");
	EditorInfo	ei;
	Info.EditorControl (ECTL_GETINFO, &ei);
	fwrite (&vblock, 1, sizeof (int), file);
	fwrite (&ei.TotalLines, 1, sizeof (int), file);
	for (int i = 0; i < ei.TotalLines; i++)
	{
		EditorGetString egs;
		EditorGetStr (&egs, i);

		int len;
		len = egs.StringLength;
		fwrite (&len, 1, sizeof (int), file);
		fwrite (egs.StringText, len, sizeof (char), file);
	}

	fclose (file);
}

static void EditorSaveRestore (void)
{
	int					vblock;
	int					sellength = 0;
	EditorInfo	ei;
	Info.EditorControl (ECTL_GETINFO, &ei);

	TEditorPos	ep = EditorGetPos ();
	char				filename[NM];
	lstrcpy (filename, TEMP_TT_TMP);
	FSF.ExpandEnvironmentStr (filename, filename, sizeof (filename));

	FILE	*file = fopen (filename, "rb");
	if (!file) return ;
	fread (&vblock, 1, sizeof (int), file);
	fread (&sellength, 1, sizeof (int), file);
	for (int i = 0; i < sellength; i++)
	{
		if ((vblock == 0) || (i == 0))
		{
			if (i != 0) Info.EditorControl (ECTL_INSERTSTRING, 0);
		}
		else
		{
			ep.Row++;
			if (ep.Row >= ei.TotalLines) Info.EditorControl (ECTL_INSERTSTRING, 0);
			EditorSetPos (ep);
		}

		int len;
		fread (&len, 1, sizeof (int), file);

		EditorGetString egs;
		EditorGetStr (&egs);

		EditorInfo	einfo;
		Info.EditorControl (ECTL_GETINFO, &einfo);

		char				*buffer = new char[egs.StringLength + len];
		char				*pDst = buffer;
		char				*pDstPos = buffer + einfo.CurPos;
		const char	*pSrc = egs.StringText;
		const char	*pSrcEnd = egs.StringText + egs.StringLength;

		while (pDst != pDstPos) *pDst++ = *pSrc++;
		pDst += fread (pDst, sizeof (char), len, file);
		while (pSrc != pSrcEnd) *pDst++ = *pSrc++;

		EditorSetString ess;
		ess.StringNumber = -1;
		ess.StringText = buffer;
		ess.StringEOL = /* FAR API defect workaround */ const_cast<char *>(egs.StringEOL);
		ess.StringLength = egs.StringLength + len;
		Info.EditorControl (ECTL_SETSTRING, &ess);

		EditorSetPos (-1, einfo.CurPos + len);

		delete[] buffer;
	}

	fclose (file);
}

static void EditorSaveRemove (void)
{
	char	filename[NM];
	lstrcpy (filename, TEMP_TT_TMP);
	FSF.ExpandEnvironmentStr (filename, filename, sizeof (filename));
	unlink (filename);
}

static char *Editor2OEM (EditorInfo *ei, char *str)
{
	if (ei)
	{
		if (ei->TableNum != -1 || ei->AnsiMode)
		{
			EditorConvertText ect;
			ect.Text = str;
			ect.TextLength = lstrlen (str);
			Info.EditorControl (ECTL_EDITORTOOEM, &ect);
		}
	}

	return (str);
}

static void EditorSetPosEx (EditorInfo *ei, int line, int initCol, const char *colSearch)
{
	int col = (initCol < 0) ? -1 : initCol;
	EditorSetPos (line, col);

	TEditorPos	pos = EditorGetPos ();
	if (pos.Row == pos.TopRow)
	{
		int tl = line - 5;
		if (tl < 1) tl = 1;
		EditorSetPos (line, -1, tl);
	}

	EditorGetString gs;
	EditorGetStr (&gs);
	if ((initCol == -2) && *colSearch)
	{
		char	*s = (char *) malloc (gs.StringLength + 1);
		if (s)
		{
			lstrcpyn (s, gs.StringText, gs.StringLength + 1);

			char	*found = lstrstr (Editor2OEM (ei, s), colSearch);
			if (found)
			{
				int newCol = found - s;
				if (newCol != col) EditorSetPos (-1, newCol);
			}

			free (s);
		}
	}

	pos = EditorGetPos ();
	if (pos.Col > gs.StringLength) EditorSetPos (-1, gs.StringLength);
}

static inline void EditorInsertText (char *s)
{
	Info.EditorControl (ECTL_INSERTTEXT, (void *) s);
}

static inline void redraw (void)
{
	Info.EditorControl (ECTL_REDRAW, 0);
}

static inline void insIns (char *ins, int &insPos)
{
	if (*ins)
	{
		EditorInsertText (ins);
		insPos = *ins = 0;
		redraw ();
	}
}

static void selectBlock (int X1, int Y1, int X2, int Y2)
{
	EditorSelect	es;
	es.BlockType = BTYPE_STREAM;
	es.BlockStartLine = min (Y2, Y1);
	es.BlockStartPos = min (X1, X2);
	es.BlockHeight = max (Y1, Y2) - min (Y1, Y2) + 1;
	es.BlockWidth = max (X1, X2) - min (X1, X2);
	Info.EditorControl (ECTL_SELECT, (void *) &es);
}

static char *GetFilePathName ()
{
	static EditorInfo ei;
	Info.EditorControl (ECTL_GETINFO, &ei);
	return (char *) ei.FileName;
}

static void EditorProcessKey (char ascii)
{
	INPUT_RECORD	tr;
	tr.EventType = KEY_EVENT;
	tr.Event.KeyEvent.bKeyDown = true;
	tr.Event.KeyEvent.wRepeatCount = 1;
	tr.Event.KeyEvent.wVirtualKeyCode = 0;
	tr.Event.KeyEvent.uChar.AsciiChar = ascii;
	tr.Event.KeyEvent.dwControlKeyState = 0;
	Info.EditorControl (ECTL_PROCESSINPUT, &tr);
}

static inline void EditorProcessFARKey (int key)
{
	Info.EditorControl (ECTL_PROCESSKEY, (void *) key);
}

static inline void EditorProcessGlobalFARKey (int key)
{
	DWORD				newkey = key;
	KeySequence ks;
	ks.Flags = KSFLAGS_DISABLEOUTPUT;
	ks.Count = 1;
	ks.Sequence = &newkey;
	Info.AdvControl (Info.ModuleNumber, ACTL_POSTKEYSEQUENCE, (void *) &ks);
}

static void EditorProcessReturnKey (int before = -1, int after = -1)
{
	INPUT_RECORD	tr;
	tr.EventType = KEY_EVENT;
	tr.Event.KeyEvent.bKeyDown = true;
	tr.Event.KeyEvent.wRepeatCount = 1;
	tr.Event.KeyEvent.wVirtualKeyCode = VK_RETURN;
	tr.Event.KeyEvent.dwControlKeyState = 0;
	if (before != -1) pluginBusy = before;
	Info.EditorControl (ECTL_PROCESSINPUT, &tr);
	if (after != -1) pluginBusy = after;
}
