struct TExec
{
	TExec();
	TExec (const TExec &);
	char			title[MAX_REG_LEN], cmd[MAX_STR_LEN], compiler[MAX_STR_LEN];
	char			enable[NM], disable[NM];
	TJumpType jumpType;
	TSaveType saveType;
	TCDType		cd;
	bool			searchBase, echo;
	void defaults (void)
	{
		*title = *cmd = *compiler = *enable = *disable = 0;
		saveType = saCurrent;
		jumpType = jtSmart;
		cd = cdNone;
		searchBase = false;
		echo = true;
	}
};

TExec::TExec ()
{
	defaults ();
}

TExec::TExec (const TExec &e)
{
	lstrcpy (title, e.title);
	lstrcpy (cmd, e.cmd);
	lstrcpy (compiler, e.compiler);
	lstrcpy (enable, e.enable);
	lstrcpy (disable, e.disable);
	jumpType = e.jumpType;
	saveType = e.saveType;
	cd = e.cd;
	echo = e.echo;
	searchBase = e.searchBase;
}

static char cmd[MAX_STR_LEN];

static char *makeCmdLine
(
	bool				ask,
	char				*buffer,
	const char	*mask,
	const char	*fn,
	const char	*title = NULL,
	const char	macro = '='
)
{
	static char n1[NM], drv[NM], dir1[NM], fil1[NM], ext1[NM];
	static char n2[NM], tmp[NM], dir2[NM], fil2[NM], ext2[NM];
	char				*i, *j, *e1 = ext1, *e2 = ext2, *smartQuote = NULL;
	fExpand (lstrcpy (tmp, fn));
	GetFullPathName (tmp, sizeof n1, n1, &j);
	fnSplit (n1, drv, dir1, fil1, ext1);
	GetShortPathName (tmp, n2, sizeof n2);
	fnSplit (n2, tmp, dir2, fil2, ext2);
	lstrcat (lstrcat (lstrcat (lstrcpy (n1, drv), dir1), fil1), ext1);
	lstrcat (lstrcat (lstrcat (lstrcpy (n2, drv), dir2), fil2), ext2);
	if (*e1 == '.') e1++;
	if (*e2 == '.') e2++;
	i = (char *) mask;
	if (ask)
	{
		nullUserStrings ();
		if (!scanUserInput (false, '=', mask, title)) return (NULL);
	}

	*(j = buffer) = 0;
	while (*i)
	{
		int		dosName = 0, noDrive = 0, noBackslash = 0, doQuote = 1;
		char	*m = lstrchr (i, macro);
		if (m)
		{
			*m = 0;
			lstrcat (j, i);
			j = lstrchr (j, 0);
			*m = macro;
			if (*++m == macro)
			{
				j[0] = macro;
				j[1] = 0;
				j = lstrchr (j, 0);
				i = m + 1;
			}
			else if (ask && lstrchr ("0123456789?", *m))
			{
				if (*m != '?') lstrcpy (j, userString[*m - '0']);
				j = lstrchr (j, 0);
				if (*m == '?')
				{
					m++;
					while (*m)
					{
						if ((m[0] == '\\') && ((m[1] == '\\') || (m[1] == '\'') || (m[1] == '?')))
						{
							m++;
							m++;
						}
						else
						{
							if (*m == '?')
								break;
							else
								m++;
						}
					}
				}

				i = m + 1;
			}
			else
			{
				int modifyer = 1;
				while (modifyer)
				{
					switch (*m)
					{
					case 'd':
						dosName = 1;
						break;
					case 'm':
						noDrive = 1;
						break;
					case 't':
						noBackslash = 1;
						break;
					case 'o':
						doQuote = 0;
						break;
					default:
						m--;
						modifyer = 0;
						break;
					}

					m++;
				}

				switch (*m)
				{
				case 'M':
					lstrcat (j, drv);
					break;
				case 'N':
					lstrcat (j, quote (doQuote, lstrcpy (tmp, dosName ? fil2 : fil1)));
					break;
				case 'E':
					if (*(m + 1) == 's')
					{
						EditorGetString egs;
						EditorGetStr (&egs);
						lstrcpyn (j + lstrlen (j), egs.StringText, egs.StringLength + 1);
						m++;
					}
					else if (*(m + 1) == 'p')
					{
						TEditorPos	pos = EditorGetPos ();
						FSF.itoa (pos.Row + 1, tmp, 10);
						lstrcat (j, tmp);
						m++;
					}
					else if (*(m + 1) == 'c')
					{
						TEditorPos	pos = EditorGetPos ();
						FSF.itoa (pos.Col + 1, tmp, 10);
						lstrcat (j, tmp);
						m++;
					}
					else
						lstrcat (j, quote (doQuote, lstrcpy (tmp, dosName ? e2 : e1)));
					break;
				case 'F':
					lstrcpy (tmp, dosName ? fil2 : fil1);
					if (*(dosName ? e2 : e1)) lstrcat (lstrcat (tmp, "."), dosName ? e2 : e1);
					lstrcat (j, quote (doQuote, tmp));
					break;
				case 'D':
					if (noDrive)
						*tmp = 0;
					else
						lstrcpy (tmp, drv);
					lstrcat (tmp, dosName ? dir2 : dir1);
					if (noBackslash) delBackslash (tmp);
					lstrcat (j, quote (doQuote, tmp));
					break;
				case 'P':
					if (noDrive)
					{
						lstrcpy (tmp, dosName ? dir2 : dir1);
						lstrcat (tmp, dosName ? fil2 : fil1);
						if (*(dosName ? e2 : e1)) lstrcat (lstrcat (tmp, "."), dosName ? e2 : e1);
						lstrcat (j, quote (doQuote, tmp));
					}
					else
						lstrcat (j, quote (doQuote, lstrcpy (tmp, dosName ? n2 : n1)));
					break;
				case '\'':
					if (smartQuote)
					{
						for (char *p = smartQuote; *p; p++)
							if (*p == '\"') *p = '\'';
						quote (1, smartQuote);
						smartQuote = NULL;
					}
					else
						smartQuote = lstrchr (j, 0);
					break;
				}

				i = m + 1;
				j = lstrchr (j, 0);
			}
		}
		else
			break;
	}

	lstrcat (j, i);
	return (buffer);
}

static void execute (TCollection *coll, const char *title, bool showExec)
{
	HANDLE	hScreen = Info.SaveScreen (0, 0, -1, -1);
	if (!showExec)
	{
		const char	*MsgItems[] = { GetMsg (MTitle), title };
		Info.Message (Info.ModuleNumber, 0, NULL, MsgItems, dimOf (MsgItems), 0);
	}

	ExecConsoleApp (cmd, coll, true, showExec);
	if (showExec) Info.Control (NULL, FCTL_SETUSERSCREEN, NULL);
	Info.RestoreScreen (hScreen);
	redraw ();
}
