struct TNavy
{
	TNavy();
	char	mask[MAX_REG_LEN], path[MAX_STR_LEN], suffixes[MAX_STR_LEN];
	int		pos, rect[4];
	bool	viewer;
};

TNavy::TNavy ()
{
	lstrcpy (this->mask, "");
	lstrcpy (this->path, "");
	lstrcpy (this->suffixes, "");
	this->pos = 0;
	this->rect[0] = -1;
	this->rect[1] = -1;
	this->rect[2] = -1;
	this->rect[3] = -1;
	this->viewer = false;
}

static void NavyInsert (const char *file, const char *suffixes, const int rect[4], bool viewer, TCollection &coll)
{
	struct nav
	{
		char	*file;
		int		rect[4];
		bool	viewer;
	};

	const char	*cursuffixes = suffixes;
	char				tmp[2] = "x";
	char				filename[2 * _MAX_PATH + 1];
	char				filenamex[2 * _MAX_PATH + 1];
	lstrcpy (filename, file);
	GetFullPathName (filename, 2 * _MAX_PATH + 1, filenamex, NULL);
	lstrcpy (filename, filenamex);
	do
	{
		if (IsFile (filename))
		{
			for (int i = 0; i < coll.getCount (); i++)
				if (lstrcmpi (((nav *) (coll[i]))->file, filename) == 0) return ;

			nav *tmp = new nav;
			tmp->rect[0] = rect[0];
			tmp->rect[1] = rect[1];
			tmp->rect[2] = rect[2];
			tmp->rect[3] = rect[3];
			tmp->viewer = viewer;
			tmp->file = new char[lstrlen (filename) + 1];
			lstrcpy (tmp->file, filename);
			coll.insert (tmp);
		}

		lstrcpy (filename, (*cursuffixes) ? file : "");
		while (*cursuffixes)
		{
			if ((*cursuffixes == ';') || (*cursuffixes == '\0'))
			{
				cursuffixes++;
				break;
			}
			else
			{
				tmp[0] = *cursuffixes;
				lstrcat (filename, tmp);
				GetFullPathName (filename, 2 * _MAX_PATH + 1, filenamex, NULL);
				lstrcpy (filename, filenamex);
				cursuffixes++;
			}
		}
	} while (*filename);
}

static void NavyFind (const char *file, const char *suffixes, const int rect[4], bool viewer, TCollection &coll)
{
	char	testFile[2 * NM];
	int		n = Info.AdvControl (Info.ModuleNumber, ACTL_GETWINDOWCOUNT, NULL);
	if (IsFile (file))
	{
		GetCurrentDirectory (sizeof (testFile), testFile);
		FSF.AddEndSlash (testFile);
		lstrcat (testFile, file);
		NavyInsert (testFile, suffixes, rect, viewer, coll);
	}

	for (int i = 0; i < n; i++)
	{
		WindowInfo	wi;
		wi.Pos = i;
		Info.AdvControl (Info.ModuleNumber, ACTL_GETWINDOWINFO, (void *) &wi);
		if (wi.Type == WTYPE_EDITOR)
		{
			char	*p = (char *) lstrrchr (lstrcpy (testFile, wi.Name), '\\');
			if (p)
			{
				lstrcpy (p + 1, file);
				NavyInsert (testFile, suffixes, rect, viewer, coll);
			}
		}
	}
}

static void NavyCollect
(
	const char	*filename,
	const char	*path,
	const char	*suffixes,
	const int		rect[4],
	bool				viewer,
	TCollection &coll
)
{
	char				curdir[2 * _MAX_PATH + 1];
	char				curfile[2 * _MAX_PATH + 1];
	char				*tmpDir = curdir;
	const char	*tmpPath = path;

	NavyInsert (filename, suffixes, rect, viewer, coll);
	while (*tmpPath)
	{
		if (*tmpPath == ';')
		{
			*tmpDir = '\0';
			*tmpPath++;
			tmpDir = curdir;
			if ((curdir[0] == '.') && (curdir[1] == '\0'))
				NavyFind (filename, suffixes, rect, viewer, coll);
			else
			{
				lstrcpy (curfile, curdir);
				lstrcat (curfile, "\\");
				lstrcat (curfile, filename);
				NavyInsert (curfile, suffixes, rect, viewer, coll);
			}
		}
		else
			*tmpDir++ = *tmpPath++;
	}

	*tmpDir = '\0';
	tmpDir = curdir;
	if ((curdir[0] == '.') && (curdir[1] == '\0'))
		NavyFind (filename, suffixes, rect, viewer, coll);
	else
	{
		lstrcpy (curfile, curdir);
		lstrcat (curfile, "\\");
		lstrcat (curfile, filename);
		NavyInsert (curfile, suffixes, rect, viewer, coll);
	}
}

static void Navigate (TLang *lng, char *realLine, DWORD pos, DWORD start, DWORD end)
{
	char				filename[2 * _MAX_PATH + 1];
	TCollection found;
	for (int i = 0; i < lng->navyColl.getCount (); i++)
	{
		TNavy *nav = (TNavy *) (lng->navyColl[i]);
		int		bounds[1][2] = { { 0, 0 } };
		if (start != -1)
		{
			int len = min (end - start + 1, _MAX_PATH);
			lstrcpyn (filename, realLine + start, end - start + 1);
			NavyCollect (filename, nav->path, nav->suffixes, nav->rect, nav->viewer, found);
		}
		else if (strMatch (realLine, nav->mask, "/^\\s*", (lng->ignoreCase ? "\\s*$/i" : "\\s*$/"), 1, bounds, &nav->pos))
		{
			if ((pos >= bounds[0][0]) && (pos <= bounds[0][1]))
			{
				int len = min (bounds[0][1] - bounds[0][0] + 1, _MAX_PATH);
				lstrcpyn (filename, realLine + bounds[0][0], bounds[0][1] - bounds[0][0] + 1);
				NavyCollect (filename, nav->path, nav->suffixes, nav->rect, nav->viewer, found);
			}
		}
	}

	struct nav
	{
		char	*file;
		int		rect[4];
		bool	viewer;
	};

	FarMenuItemEx *mMenu = NULL;
	if (found.getCount () > 1) mMenu = new FarMenuItemEx[found.getCount ()];
	for (int i = 0; ((i < found.getCount ()) && (found.getCount () > 1)); i++)
	{
		mMenu[i].Flags = MIF_USETEXTPTR;
		mMenu[i].Text.TextPtr = ((nav *) (found[i]))->file;
	}

	int BreakCode;
	int BreakKeys[2] = { VK_F3, 0 };
	int ExitCode = 0;
	while (found.getCount () > 1)
	{
		ExitCode = Info.Menu
			(
				Info.ModuleNumber,
				-1,
				-1,
				0,
				FMENU_WRAPMODE | FMENU_USEEXT,
				GetMsg (MNavyMenu),
				GetMsg (MNavyMenuBottom),
				NULL,
				BreakKeys,
				&BreakCode,
				(const FarMenuItem *) mMenu,
				found.getCount ()
			);
		if (ExitCode >= 0)
		{
			if (BreakCode == 0)
			{
				const char	*Msg[5];
				Msg[0] = GetMsg (MNavigation);
				Msg[1] = "\1";
				Msg[2] = ((nav *) (found[ExitCode]))->file;
				Msg[3] = "\1";
				Msg[4] = GetMsg (MOK);
				Info.Message (Info.ModuleNumber, FMSG_LEFTALIGN, NULL, Msg, sizeof (Msg) / sizeof (Msg[0]), 1);
				continue;
			}
			else
				break;
		}
		else
			break;
	}

	if (found.getCount () > 1) delete[] mMenu;
	if ((found.getCount ()) && (ExitCode >= 0))
	{
		CONSOLE_SCREEN_BUFFER_INFO	ConBuffInfo;
		HANDLE	ConHnd = GetStdHandle (STD_OUTPUT_HANDLE);
		GetConsoleScreenBufferInfo (ConHnd, &ConBuffInfo);

		char	navRectLeft[32];
		char	navRectRight[32];
		char	navRectTop[32];
		char	navRectBottom[32];
		GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectLeft", navRectLeft, "0", 32);
		GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectTop", navRectTop, "50", 32);
		GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectRight", navRectRight, "100", 32);
		GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectBottom", navRectBottom, "100", 32);

		WORD	x1, y1, x2, y2;
		x1 = ((((nav *) (found[ExitCode]))->rect[0] >= 0) && (((nav *) (found[ExitCode]))->rect[0] <= 100)) ? ((nav *) (found[ExitCode]))->rect[0] : FSF.atoi (navRectLeft);
		y1 = ((((nav *) (found[ExitCode]))->rect[1] >= 0) && (((nav *) (found[ExitCode]))->rect[1] <= 100)) ? ((nav *) (found[ExitCode]))->rect[1] : FSF.atoi (navRectTop);
		x2 = ((((nav *) (found[ExitCode]))->rect[2] >= 0) && (((nav *) (found[ExitCode]))->rect[2] <= 100)) ? ((nav *) (found[ExitCode]))->rect[2] : FSF.atoi (navRectRight);
		y2 = ((((nav *) (found[ExitCode]))->rect[3] >= 0) && (((nav *) (found[ExitCode]))->rect[3] <= 100)) ? ((nav *) (found[ExitCode]))->rect[3] : FSF.atoi (navRectBottom);
		x1 = ConBuffInfo.dwSize.X * x1 / 100;
		y1 = ConBuffInfo.dwSize.Y * y1 / 100;
		x2 = ConBuffInfo.dwSize.X * x2 / 100;
		y2 = ConBuffInfo.dwSize.Y * y2 / 100;
		if (((nav *) (found[ExitCode]))->viewer)
		{
			Info.Viewer
				(
					((nav *) (found[ExitCode]))->file,
					NULL,
					x1,
					y1,
					x2,
					y2,
					VF_NONMODAL | VF_IMMEDIATERETURN | VF_DISABLEHISTORY | VF_ENABLE_F6
				);
		}
		else
		{
			Info.Editor
				(
					((nav *) (found[ExitCode]))->file,
					NULL,
					x1,
					y1,
					x2,
					y2,
					EF_NONMODAL | EF_IMMEDIATERETURN | EF_DISABLEHISTORY,
					0,
					1
				);
		}
	}

	for (int i = 0; i < found.getCount (); i++) delete[]((nav *) (found[i]))->file;
	found.removeAll ();
}

static void SelectNavigation (TEInfo *te)
{
	if (te)
	{
		TLang *lng = (TLang *) (langColl[te->lang]);
		if (lng)
		{
			char				line[MAX_STR_LEN];
			EditorInfo	ei;
			Info.EditorControl (ECTL_GETINFO, &ei);

			EditorGetString gs;
			TEditorPos			epos = EditorGetPos ();
			EditorGetStr (&gs);
			Editor2OEM (&ei, lstrcpyn (line, gs.StringText, gs.StringLength + 1));
			Navigate (lng, line, epos.Col, gs.SelStart, gs.SelEnd);
		}
	}
}

static void SelectNavigationList (TEInfo *te)
{
	if (te)
	{
		TLang *lng = (TLang *) (langColl[te->lang]);
		if (lng)
		{
			char				line[MAX_STR_LEN];
			char				filename[2 * _MAX_PATH + 1];
			TCollection found;
			EditorInfo	ei;
			Info.EditorControl (ECTL_GETINFO, &ei);
			for (int l = 0; l < ei.TotalLines; l++)
			{
				EditorGetString egs;
				EditorGetStr (&egs, l);
				lstrcpyn (line, egs.StringText, egs.StringLength + 1);
				for (int i = 0; i < lng->navyColl.getCount (); i++)
				{
					TNavy *nav = (TNavy *) (lng->navyColl[i]);
					int		bounds[1][2] = { { 0, 0 } };
					if (strMatch (line, nav->mask, "/^\\s*", (lng->ignoreCase ? "\\s*$/i" : "\\s*$/"), 1, bounds, &nav->pos))
					{
						int len = min (bounds[0][1] - bounds[0][0] + 1, _MAX_PATH);
						lstrcpyn (filename, line + bounds[0][0], len);
						NavyCollect (filename, nav->path, nav->suffixes, nav->rect, nav->viewer, found);
					}
				}
			}

			struct nav
			{
				char	*file;
				int		rect[4];
				bool	viewer;
			};

			FarMenuItemEx *mMenu = NULL;
			if (found.getCount () > 1) mMenu = new FarMenuItemEx[found.getCount ()];
			for (int i = 0; ((i < found.getCount ()) && (found.getCount () > 1)); i++)
			{
				mMenu[i].Flags = MIF_USETEXTPTR;
				mMenu[i].Text.TextPtr = ((nav *) (found[i]))->file;
			}

			int BreakCode;
			int BreakKeys[2] = { VK_F3, 0 };
			int ExitCode = 0;
			while (found.getCount () > 1)
			{
				ExitCode = Info.Menu
					(
						Info.ModuleNumber,
						-1,
						-1,
						0,
						FMENU_WRAPMODE | FMENU_USEEXT,
						GetMsg (MNavyMenu),
						GetMsg (MNavyMenuBottom),
						NULL,
						BreakKeys,
						&BreakCode,
						(const FarMenuItem *) mMenu,
						found.getCount ()
					);
				if (ExitCode >= 0)
				{
					if (BreakCode == 0)
					{
						const char	*Msg[5];
						Msg[0] = GetMsg (MNavigation);
						Msg[1] = "\1";
						Msg[2] = ((nav *) (found[ExitCode]))->file;
						Msg[3] = "\1";
						Msg[4] = GetMsg (MOK);
						Info.Message (Info.ModuleNumber, FMSG_LEFTALIGN, NULL, Msg, sizeof (Msg) / sizeof (Msg[0]), 1);
						continue;
					}
					else
						break;
				}
				else
					break;
			}

			if (found.getCount () > 1) delete[] mMenu;
			if ((found.getCount ()) && (ExitCode >= 0))
			{
				CONSOLE_SCREEN_BUFFER_INFO	ConBuffInfo;
				HANDLE	ConHnd = GetStdHandle (STD_OUTPUT_HANDLE);
				GetConsoleScreenBufferInfo (ConHnd, &ConBuffInfo);

				char	navRectLeft[32];
				char	navRectRight[32];
				char	navRectTop[32];
				char	navRectBottom[32];
				GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectLeft", navRectLeft, "0", 32);
				GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectTop", navRectTop, "50", 32);
				GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectRight", navRectRight, "100", 32);
				GetRegKey (HKEY_CURRENT_USER, "", "NavigationRectBottom", navRectBottom, "100", 32);

				WORD	x1, y1, x2, y2;
				x1 = ((((nav *) (found[ExitCode]))->rect[0] >= 0) && (((nav *) (found[ExitCode]))->rect[0] <= 100)) ? ((nav *) (found[ExitCode]))->rect[0] : FSF.atoi (navRectLeft);
				y1 = ((((nav *) (found[ExitCode]))->rect[1] >= 0) && (((nav *) (found[ExitCode]))->rect[1] <= 100)) ? ((nav *) (found[ExitCode]))->rect[1] : FSF.atoi (navRectTop);
				x2 = ((((nav *) (found[ExitCode]))->rect[2] >= 0) && (((nav *) (found[ExitCode]))->rect[2] <= 100)) ? ((nav *) (found[ExitCode]))->rect[2] : FSF.atoi (navRectRight);
				y2 = ((((nav *) (found[ExitCode]))->rect[3] >= 0) && (((nav *) (found[ExitCode]))->rect[3] <= 100)) ? ((nav *) (found[ExitCode]))->rect[3] : FSF.atoi (navRectBottom);
				x1 = ConBuffInfo.dwSize.X * x1 / 100;
				y1 = ConBuffInfo.dwSize.Y * y1 / 100;
				x2 = ConBuffInfo.dwSize.X * x2 / 100;
				y2 = ConBuffInfo.dwSize.Y * y2 / 100;
				if (((nav *) (found[ExitCode]))->viewer)
				{
					Info.Viewer
						(
							((nav *) (found[ExitCode]))->file,
							NULL,
							x1,
							y1,
							x2,
							y2,
							VF_NONMODAL | VF_IMMEDIATERETURN | VF_DISABLEHISTORY | VF_ENABLE_F6
						);
				}
				else
				{
					Info.Editor
						(
							((nav *) (found[ExitCode]))->file,
							NULL,
							x1,
							y1,
							x2,
							y2,
							EF_NONMODAL | EF_IMMEDIATERETURN | EF_DISABLEHISTORY,
							0,
							1
						);
				}
			}

			for (int i = 0; i < found.getCount (); i++) delete[]((nav *) (found[i]))->file;
			found.removeAll ();
		}
	}
}
