static void parseExec (TExec *tmpe, const char *name, const char *value)
{
	if (!FSF.LStricmp (name, "Save"))
	{
		if (!FSF.LStricmp (value, "Current"))
			tmpe->saveType = saCurrent;
		else if (!FSF.LStricmp (value, "All"))
			tmpe->saveType = saAll;
		else
			tmpe->saveType = saNone;
	}
	else if (!FSF.LStricmp (name, "CD"))
	{
		if (!FSF.LStricmp (value, "File"))
			tmpe->cd = cdFile;
		else if (!FSF.LStricmp (value, "Base"))
			tmpe->cd = cdBase;
		else
			tmpe->cd = cdNone;
	}
	else if (!FSF.LStricmp (name, "Jump"))
	{
		if (!FSF.LStricmp (value, "Smart"))
			tmpe->jumpType = jtSmart;
		else if (!FSF.LStricmp (value, "Menu"))
			tmpe->jumpType = jtMenu;
		else if (!FSF.LStricmp (value, "First"))
			tmpe->jumpType = jtFirst;
		else
			tmpe->jumpType = jtNone;
	}
	else if (!FSF.LStricmp (name, "Echo"))
		tmpe->echo = FSF.atoi (value) ? true : false;
	else if (!FSF.LStricmp (name, "Title"))
		lstrcpy (tmpe->title, value);
	else if (!FSF.LStricmp (name, "Enable"))
	{
		if (value[0] == '*' && value[1] == '\\')
		{
			lstrcpy (tmpe->enable, value);
			tmpe->searchBase = true;
		}
		else
			lstrcpy (tmpe->enable, value + 2);
	}
	else if (!FSF.LStricmp (name, "Disable"))
		lstrcpy (tmpe->disable, value);
	else if (!FSF.LStricmp (name, "Compiler"))
		lstrcpy (tmpe->compiler, value);
}

static void parseCompiler (TCompiler *tmpc, const char *name, const char *value)
{
	if (!FSF.LStricmp (name, "Error"))
		lstrcpy (tmpc->err, value);
	else if (!FSF.LStricmp (name, "Line"))
		tmpc->line = FSF.atoi (value);
	else if (!FSF.LStricmp (name, "File"))
		tmpc->fileMatch = FSF.atoi (value);
	else if (!FSF.LStricmp (name, "Col"))
	{
		char	*v = (char *) value;
		if (*value == '?')
		{
			v++;
			tmpc->searchCol = true;
		}

		tmpc->col = FSF.atoi (v);
	}
}

static char *parseItem (TCollection *dc, char * &line, char *kwd, char *value)
{
	if (IsCharAlpha (*skipSpaces (line)))
	{
		char	*k = kwd;
		while (*line && IsCharAlpha (*line)) *k++ = *line++;
		*k = *value = 0;
		if (*skipSpaces (line) == '=')
		{
			line++;
			if (!dc)
			{
				getWord (line, value);
			}
			else
			{
				char	buf[MAX_STR_LEN], *pv = value;
				getWord (line, buf);
				for (char *pch = buf; *pch;)
				{
					if ('&' != *pch)
					{
						*pv++ = *pch++;
					}
					else
					{
						++pch;						//skip '&'
						char	name[MAX_STR_LEN], *pname = name;
						while (*pch && *pch != ';') *pname++ = *pch++;
						*pname = 0;

						//if (!*pch) complain about entity reference syntax;
						if (*pch) ++pch;	//skip
															///';
															///'
						for (size_t i = 0; i < dc->getCount (); ++i)
						{
							TDefine *d = (TDefine *) ((*dc)[i]);
							if (0 == lstrcmp (d->name, name))
							{
								for (const char *pdv = d->value; *pdv;) *pv++ = *pdv++;
								break;
							}
						}

						//if (i == dc->getCount()) complain about undefined entity;
					}
				}

				*pv = 0;
			}
		}

		return (line);
	}

	return (NULL);
}

static char *getItem (char * &line, char *kwd, bool &group)
{
	group = true;

	bool	coment = true;
	while (coment)
	{
		if (*skipSpaces (line) == '<')
		{
			if (line[1] == '!')
			{
				if (line[2] == '-')
				{
					if (line[3] == '-')
					{
						line += 4;
						coment = true;
						while (*line)
						{
							if (line[0] == '-')
								if (line[1] == '-')
									if (line[2] == '>')
									{
										line += 3;
										break;
									}

							line++;
						}
					}
					else
						coment = false;
				}
				else
					coment = false;
			}
			else
				coment = false;
		}
		else
			coment = false;
	}

	if (*skipSpaces (line) == '<')
	{
		char	*stLine = line;
		char	*k = kwd;
		line++;
		if (*line && (*line == '/')) *k++ = *line++;
		while (*line && IsCharAlpha (*line)) *k++ = *line++;
		*k = 0;
		k = --line;

		int inQuote = 0;
		while (*++line)
		{
			if (inQuote)
			{
				if (*line == '\\' && line[1] == '\"')
					line++;
				else if (*line == '\"')
					inQuote = 0;
			}
			else
			{
				if (*line == '>')
				{
					if (line > stLine)
					{
						--line;
						if (*line == '/')
						{
							*line = 0;
							group = false;
						}

						line++;
					}

					*line = 0;
					line++;
					break;
				}
				else if (*line == '\"')
					inQuote = 1;
			}
		}

		return (k);
	}

	return (NULL);
}

static void findSectionInXML (char * &p)
{
	bool	group;
	char	name[MAX_STR_LEN];
	while ((getItem (p, name, group)) != NULL)
		if (group && !FSF.LStricmp (name, "TrueTpl")) break;
}

static char *GetXmlParam (char **szParam)
{
	static char szString[_MAX_PATH + 1];
	char				*p = *szParam;
	if (*(++p) == '\'')
	{
		char	*o = szString;
		while (*++p)
		{
			if ((*p == '\\') && ((p[1] == '\\') || (p[1] == '\'')))
				p++;
			else if (*p == '\'')
				break;
			if (*p) *o++ = *p;
		}

		*o = 0;
		*szParam = p;
		return (szString);
	}
	else
		return (NULL);
}
